
;****************************************************************************************
;*                           POWERBANK VON DL3UKH                                       *
;*                           ====================                                       *
;*                                 27.08.2022                                           *
;*--------------------------------------------------------------------------------------*
;* ALS PIC-IC WURDE GENOMMEN PIC12F629-I/P                                              *
;* DER OSZILLATOR IST EIN INTERNER OSZILLATOR MIT 4MHZ.  (T=0.25SEC)                   *
;*                                                                                      *
;****************************************************************************************
 
        LIST    P=12F629       
        TITLE   "POWERBANK"

         ERRORLEVEL -302
         ERRORLEVEL -207

         INCLUDE <P12F629.INC>

;KONFIGURATIONS-EINSTELLUNGEN FUER DAS BRENNEN
 __CONFIG _INTRC_OSC_NOCLKOUT & _WDT_OFF & _PWRTE_OFF & _BODEN_ON & _MCLRE_OFF & _CPD_OFF & _CP_OFF


;--------------------------------------------------------------------------
;REGISTER ZUORDNEN NACH DEM DIE FESTEN-REGISTER ZU ENDE SIND
;DIES IST ALSO DIE RAM-SPEICHER ZUORDNUNG
HELP                    EQU     0X22                ;FUER ZEITSCHLEIFE 
HELP1                   EQU     0X23                ;FUER ZEITSCHLEIFE
TAST                    EQU     0X24                ;AUS_TASTER AKTIV BIT0-HIGH  
ZAE                     EQU     0X25                ;SCHLEIFENZAEHLER
                     
SEC                     EQU     0X26                ;SEC-TAKT REALISIEREN                    
ZEIT                    EQU     0X27                ;NACH WELCHER ZEIT DIE AUTOMATISCHE LAST ZUGESCHALTET WIRD
EIN                     EQU     0X28                ;EIN_LED MERKER
                     
;FUER INTERRUPT
S_STACK                 EQU     0X29
W_STACK                 EQU     0X2A
                 
;--------------------------------------------------------------------------                 
;VARIABLEN
BIT0                    EQU     0
BIT1                    EQU     1
BIT2                    EQU     2
BIT3                    EQU     3
BIT4                    EQU     4
BIT5                    EQU     5
BIT6                    EQU     6
BIT7                    EQU     7
                    
;--------------------------------------------------------------------------
;PORTZUORDNUNG==> 
;PIN KURZ-SCHREIBWEISE              
LAST        EQU 4   ;GP4 MOSFET-ZUSCHALTUNG FUER AUTOMATISCHE LAST (ALS AUSGANG EINSTELLEN UNTEN BEI MAIN) 
LED_RT      EQU 1   ;GP1
LED_GN      EQU 0   ;GP0
HALTEND     EQU 5   ;GP5 SELBSTHALTE SCHALTUNG FUER MOSFET
AUS_TASTER  EQU 2   ;GP2 TASTER UM AUSZUSCHALTEN
  
;===============================================================================                 
;HIER KOENNTE MAN AENDERN :                      
;1.)
;IMPULSLNGE FR DIE AUTOMATISCHE LAST ZUSCHALTUNG / ZEIT-LNGE WIE LANG ZUGESCHALTET WIRD        
IMPULS_LAENGE           EQU D'22'   ;22*100MSEC=2200MSEC=2,2SEC
 
;2.)
;WIE LANG DIE PAUSE SEIN SOLL FUER AUTOATISCHE LAST ZUSCHALTUNG           
PAUSEN_LAENGE           EQU D'25'   ;DIREKT DIE SEC EINGEBEN // JETZT 25SEC
   
;ZUSAMMENFASSUNG:
;ALLER 25SEC KOMMT EIN IMPULS VON 2,2SEC LAENGE

;3.)
;MIT GRUENER EINSCHALT-LED
EIN_LED                 EQU D'1'    ;HIER BLINKT ES BEIM EINSCHALTEN UND LEUCHTET DAUERND WEITER BEI EINGESCHALTET
                                    ;MAN KANN KONTROLLIEREN OB DIE BATTERIE NOCH AUF EIN IST
                                    ;UND BRAUCHT NICHT SO LANGE WARTEN BIS DER ERHALTUNGS-IMPULS (BLAUE LED) NACH LAENGERER ZEIT AUFBLITZT
                                    
;EIN_LED                 EQU D'0'   ;HIER ERLISCHT DIE GRUENE LED NACH DEM SIE BEIM EINSCHALTEN AM ANFANG GEBLINKT HAT
                                    ;ALSO OHNE DAUERLEUCHTEN   /// MAN KANN NUR AN HAND DES AUFBLITZENS DER BLAUEN LED KONTROLLIEREN OB NOCH AUF EIN
;4.)
;WIEVIELE BLINKS SOLL ES BEIM EIN- (GN-LED) UND AUS- (RT-LED) SCHALTEN MACHEN
BLINK_ANZAHL            EQU D'7'           
           
           
;------------------------- MACROS -----------------------------------------

BANK0                   MACRO                   ;MACRO TO SELECT DATA RAM BANK 0
                        BCF     STATUS,RP0
                        ENDM



BANK1                   MACRO                   ;MACRO TO SELECT DATA RAM BANK 1
                        BSF     STATUS,RP0
                        ENDM

;------------ MACROS FUER INTERRUPT -----------------------------------------
;WENN EINE UNTERBRECHUNG VOM EIGENTLICHEN PROGRAMM KOMMT, MUSS REGISTER "W" UND "STATUS" GERETTET/GEMERKT WERDEN.
;DIES PASSIERT MITTELS RAM-SPEICHERZELLEN W_STACK UND S_STACK

PUSH    MACRO
        MOVWF   W_STACK
        MOVF    STATUS, W
        MOVWF   S_STACK
        ENDM
;----------------------------------------------------------------------------

;WENN DAS INTERRUPT-PROGRAMM ABGEARBEITET IST (ZU ENDE IST), MSSEN DIE GEMERKTEN/GERETTEN REGISTER "W" UND "STATUS"
;WIEDER HERVORGEHOLT WERDEN, UM IM EIGENTLICHEN PROGRAMM WEITER ZU ARBEITEN

POP     MACRO
        MOVF    S_STACK, W
        MOVWF   STATUS
        SWAPF   W_STACK, F
        SWAPF   W_STACK, W
        ENDM
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;SPEICHERANFANG BZW. BEGINN

                        ORG 0X0000
                        GOTO MAIN               ; SPRUNG ZUM HAUPTPROGRAMM
                        ORG 0X0004
                        GOTO INTERRUPT          ; BEI EINEM INTERRUPT SPRUNG

                        ORG 0X0010
 
;*****************************************************************************
;*                            HAUPTPROGRAMM   MIT DEM NAMEN "MAIN"           *
;*****************************************************************************
;PORT-INITIALISIERUNG
MAIN
                CLRF    GPIO     
                MOVLW   0X07                    ;COMPARATORS ALLE AUS 
                MOVWF   CMCON

                BANK1

                MOVLW   B'11001100'             ;GP0 GP1 GP4 GP5 ALS AUSGANG (ALSO HIER DIE 0), SONST ALLES EINGNGE GP3 GP2 (ALSO IMMER DIE (1))
                MOVWF   TRISIO

;KALIBRIERUNGSWERT FR DEN INTERNEN OSZILLATOR AM ENDE DES HAUPTSPEICHERS BERNEHMEN
                CALL    0X03FF                  
                MOVWF   OSCCAL
 
;INIT TIMER 0 FUER FUER ZEITSCHLEIFE 
                MOVLW   B'10000111'             ;BIT0-2 111=PRESCALER256, BIT3 0=TMR0, BIT4 0=ENTFAELLT, BIT5 0=TAKT VOM SYSTEM, BIT6 0=KEIN RB-INTERRUPT, BIT7 1=PULL-UPS DISABLE 
                MOVWF   OPTION_REG      
                BANK0
                         
                        
                MOVLW   D'5'                        ;ZEITSCHLEIFE 50MSEC ZUM AUFLADEN DER ELKOS UND KONDENSATOREN
                CALL    ZS10MSEC

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;                  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;                
;INTERRUPT GP2 INIT / ZULASSEN

                        BANK1
                        BCF     OPTION_REG, INTEDG  ;FALLENDE FLANKE ZULASSEN / ALSO VON +5VOLT AUF NULL 
                        BANK0
              
;NUR FUER GP2
                        ;BSF     INTCON, GIE        ;ALLGEMEINES INTERRUPT ENABLE 
                                                    ;ERST SPAETER AM SCHLUSS ZULASSEN

                        BSF     INTCON, PEIE        ;INTERUPT ZULASSEN PEREPHERIE 
                                                    ;WEIS NICHT OB ES DRIN SEIN MUSS / WAHRSCHEINLICH JA

                        BSF     INTCON, INTE        ;INTERRUPT EXTERNAL VON GP2 ZULASSEN

                        BCF     INTCON, GPIE        ;INTERRUPT FR EXTERNEN PIN NICHT ZULASSEN 
                                                    ;MIT CHANGE VON GP2 AUF GPX-EINGANG!!!

                        BCF     INTCON, GPIF        ;INTERRUPT-FLAG WIRD ERST BEI INTERRUPT GESETZT / JETZT LSCHEN
                        BCF     INTCON, INTF        ;INTERRUPT-FLAG WIRD ERST BEI INTERRUPT GESETZT / JETZT LSCHEN
                
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;   
;INTERRUPT GP4 INIT / ZULASSEN
                BCF     T1CON, TMR1ON   ;TIMER1 ANHALTEN
                
                BCF     PIR1, TMR1IF    ;OVERFLOW 0 MACHEN
            
;                BSF     INTCON, GIE     ;ALLGEMEINES INTERRUPT ENABLE;ERST SPAETER AM SCHLUSS ZULASSEN
;                BSF     INTCON, PEIE    ;WAR SCHON OBEN
            
                BANK1
                BSF     PIE1, TMR1IE    ;OVERFLOW INTERRUPT BEI TIMER1 ZULASSEN/ENABLE
                BANK0
            
                MOVLW   0XDC            ;500MS/32=15.625MS/0.25S=62500TAKTE
                MOVWF   TMR1L           ;0X10000-62500(0XF424)=0X0BDC  
                MOVLW   0X0B
                MOVWF   TMR1H
   
                MOVLW   D'2'
                MOVWF   SEC
                
                MOVLW   PAUSEN_LAENGE
                MOVWF   ZEIT                

                MOVLW   0X31            ;PRESCALE1:8 / SYSTEMCLOCK T/4 ==>1/32
                MOVWF   T1CON           ;START TIMER1   

                BSF     INTCON, GIE     ;ALLGEMEINES INTERRUPT ENABLE
                
                
;---------------------------------INIT ZU ENDE----------------------------------
              
                BSF     GPIO, HALTEND           ;SELBSTHALTUNG AKTIV
                
                MOVLW   BLINK_ANZAHL            ;SCHLEIFENANZAHL ALS BLINK
                MOVWF   ZAE
                
                BSF     GPIO, LAST
                CALL    STOP_ZS1
                
SCHLEIFE00      BSF     GPIO, LED_GN     
                MOVLW   D'25'                   ;ZEITSCHLEIFE 250MSEC 
                CALL    ZS10MSEC_TASTER
                BTFSC   TAST, BIT0
                GOTO    ANF01
                
                BCF     GPIO, LED_GN     
                MOVLW   D'25'                   ;ZEITSCHLEIFE 250MSEC 
                CALL    ZS10MSEC_TASTER
                BTFSC   TAST, BIT0
                GOTO    ANF01

                DECFSZ  ZAE, F
                GOTO    SCHLEIFE00
               
ANF01           MOVLW   EIN_LED                 ;DAUERLEUCHTEN NACH DEM ES GEBLINKT HAT
                MOVWF   EIN
                BTFSS   EIN, BIT0
                GOTO    ANF01X
                
                BSF     GPIO, LED_GN
                GOTO    ANF01Y
                
ANF01X          BCF     GPIO, LED_GN
                
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ANF01Y          MOVLW   IMPULS_LAENGE
                CALL    ZS100MSEC_TASTER
                BTFSC   TAST, BIT0
                GOTO    ANF02
                BCF     GPIO, LAST
                CALL    START_ZS1
                
ANF02           GOTO    ANF02
   
;*****************************************************************************
;*                            INTERRUPT                                      *
;*****************************************************************************                   
INTERRUPT          
                PUSH
                BTFSC   PIR1, TMR1IF
                GOTO    IMPULS_PAUSE
               
;AUS-SCHALTEN / BEENDEN                
                MOVLW   BLINK_ANZAHL             ;SCHLEIFENANZAHL ALS BLINK
                MOVWF   ZAE
                BCF     GPIO, LAST
                 
SCHLEIFE01      BSF     GPIO, LED_RT     
                MOVLW   D'25'                   ;ZEITSCHLEIFE 250MSEC 
                CALL    ZS10MSEC
                
                BCF     GPIO, LED_RT     
                MOVLW   D'25'                   ;ZEITSCHLEIFE 250MSEC 
                CALL    ZS10MSEC

                DECFSZ  ZAE, F
                GOTO    SCHLEIFE01
                
;NICHT MEHR NOTWENDIG                
                ;BCF     INTCON, GPIF    ;INTERRUPT-FLAG WIRD ERST BEI INTERRUPT GESETZT 
                ;BCF     INTCON, INTF    ;INTERRUPT-FLAG WIRD ERST BEI INTERRUPT GESETZT 
                                        ;JETZT AUF JEDEN FALL HIER LSCHEN
                                       
                BCF     GPIO, HALTEND           ;SELBSHALTUNG INAKTIV
                POP
                RETFIE
                
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;                
IMPULS_PAUSE
                BCF     T1CON, TMR1ON   ;TIMER1 ANHALTEN
                MOVLW   0XDC            ;500MS/32=15.625MS/0.25S=62500TAKTE
                MOVWF   TMR1L           ;0X10000-62500(0XF424)=0X0BDC
                MOVLW   0X0B
                MOVWF   TMR1H
                BCF     PIR1, TMR1IF    ;OVERFLOW 0 MACHEN

                DECFSZ  SEC, F          ;EIN SEC-TAKT DA TIMER1 ALS 0,5SEC-TAKT LAEUFT
                GOTO    X1
                
                MOVLW   D'2'
                MOVWF   SEC
                
                DECFSZ  ZEIT, F
                GOTO    X1
                
                MOVLW   PAUSEN_LAENGE
                MOVWF   ZEIT                

                BSF     GPIO, LAST
                CALL    STOP_ZS1
                
                MOVLW   IMPULS_LAENGE
                CALL    ZS100MSEC_TASTER
                BTFSC   TAST, BIT0
                GOTO    X2
                
                BCF     GPIO, LAST
                ;CALL    START_ZS1      ;SCHON UNTEN
                
X1              BSF     T1CON, TMR1ON   ;TIMER1 STARTEN
 
X2              POP
                RETFIE
    
;*****************************************************************************
;*                            ZEIT FUER AUTOMATISCHE LAST STOPPEN            *
;*****************************************************************************                   
STOP_ZS1
        BCF     T1CON, TMR1ON   ;TIMER1 ANHALTEN  GENUEGT!!!
        RETURN
     
;*****************************************************************************
;*                            ZEIT FUER AUTOMATISCHE LAST STOPPEN            *
;*****************************************************************************                   
START_ZS1
        BSF     T1CON, TMR1ON   ;TIMER1 STARTEN  GENUEGT!!!
        RETURN
     
;*****************************************************************************
;*                            ZEITSCHLEIFE 10MSEC                            *
;*****************************************************************************                   
ZS10MSEC                MOVWF   HELP
                
ZZZ01                   MOVLW   D'237'
                        MOVWF   TMR0                    
                        BCF     INTCON, T0IF            ;OVERFLOW = 0
                 
ZZZ02                   BTFSS   INTCON, T0IF            
                        GOTO    ZZZ02                    
                                                
ZZZ03                   DECFSZ  HELP, F                
                        GOTO    ZZZ01                    
                        RETURN

;ZEITBERECHNUNG FUER 10MSEC
;PRESCALER 256X  +  SYSTEM 4X  ====> 256*4*0.25SEC=256SEC
;10MSEC:256SEC=39    
;TMR0 ALSO D'256'-D'39'=D'217' ENTSPRICHT CA. 10MSEC
;217 DEZIMAL OBEN EINTRAGEN      
                        
;*****************************************************************************
;*                            ZEITSCHLEIFE 10MSEC MIT TASTER-ABFRAGE         *
;*****************************************************************************                   
ZS10MSEC_TASTER         MOVWF   HELP
                
                        BCF     TAST, BIT0
                
Z01                     MOVLW   D'237'
                        MOVWF   TMR0                    
                        BCF     INTCON, T0IF            ;OVERFLOW = 0
                 
Z02                     BTFSC   INTCON, T0IF            
                        GOTO    Z03
                        BTFSC   GPIO, AUS_TASTER
                        GOTO    Z02                    
                        BSF     TAST, BIT0
                        RETURN
                        
Z03                     DECFSZ  HELP, F                
                        GOTO    Z01                    
                        RETURN
 
;*****************************************************************************
;*                            ZEITSCHLEIFE 100MSEC                           *
;*****************************************************************************                   
ZS100MSEC_TASTER        MOVWF   HELP

                        BCF     TAST, BIT0
                
ZZ04                    MOVLW   D'10'
                        MOVWF   HELP1
                        
ZZ01                    MOVLW   D'237'
                        MOVWF   TMR0                    
                        BCF     INTCON, T0IF            ;OVERFLOW = 0
                 
ZZ02                    BTFSC   INTCON, T0IF            
                        GOTO    ZZ03
                        BTFSC   GPIO, AUS_TASTER
                        GOTO    ZZ02                    
                        BSF     TAST, BIT0
                        RETURN                    
 
ZZ03                    DECFSZ  HELP1, F
                        GOTO    ZZ01   
                        
                        DECFSZ  HELP, F
                        GOTO    ZZ04
                        RETURN

 
                    
                    
                    END
    
    
    
    
    
    
    
    
    
    
    














